/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>

#include "Boton.h"
#include "Sistema.h"

/**
 * Constructor de la clase boton a partir de posicion y tamao
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param titulo      Texto que aparece en el boton
 * @param x           Posicion horizontal del boton relativa al padre
 * @param y           Posicion vertical del boton relativa al padre
 * @param ancho       Anchura del boton
 * @param alto        Altura del boton
 */
Boton::Boton(CUADRO** cuadroPadre, Texto* titulo,
             int x, int y, int ancho, int alto)
     : Widget(cuadroPadre, x, y, ancho, alto)
{
  // Inicializar los atributos por defecto
  colorFondo = DF_COLORFONDO;
  colorBorde = DF_COLORBORDE;
  colorTexto = DF_COLORTEXTO;
  colorRaton = DF_COLORRATON;
  flags      = DF_FLAGSBOTON;

  // Preparar el texto del boton
  if (titulo->cadena){

    INFOTEXTO infoTexto;
    infoTexto.cadena = new char[strlen(titulo->cadena)+1];
    strcpy(infoTexto.cadena,titulo->cadena);
    infoTexto.x=0;
    infoTexto.y=0;
    infoTexto.colorTexto  = DF_COLORTEXTO;
    infoTexto.colorRaton  = DF_COLORRATON;
    infoTexto.fuente      = DF_FUENTEBOTON;
    infoTexto.flags       = WID_DETECTARRATON | WID_CENTRARX | WID_CENTRARY ;

    texto = new Texto(&cuadro,infoTexto);

  }
  else texto=NULL;


}

/**
 * Constructor de la clase boton a partir de un fichero de imagen
 * @param cuadroPadre   Indica cual es su cuadro padre (en general a que ventana
 *                      pertenece). Si esta a 0, el padre es la pantalla
 * @param ficheroImagen Nombre del fichero que contiene la imagen
 * @param x             Posicion horizontal del boton relativa al padre
 * @param y             Posicion vertical del boton relativa al padre
 */
Boton::Boton(CUADRO** cuadroPadre, char *ficheroImagen, int x, int y)
     : Widget(cuadroPadre, x, y, ficheroImagen)
{
  // Inicializar los atributos por defecto
  texto=NULL;
  colorFondo = DF_COLORFONDO;
  colorBorde = DF_COLORBORDE;
  colorTexto = DF_COLORTEXTO;
  colorRaton = DF_COLORRATON;
  flags      = DF_FLAGSBOTON;

};

/**
 * Destructor de la clase boton
 */
Boton::~Boton()
{
  // Libera el texto, si habia
  if (texto) { delete texto; texto=0; }
}


/**
 * Detecta si el raton pasa por encima del boton
 * Si es asi, resalta el texto con el color seleccionado
 * @return Botones del raton pulsados (Si ha habido alguno)
 */
DWORD Boton::detectarRaton()
{
  // Detectar el raton sobre el widget
  int msg = Widget::detectarRaton();

  // Resaltar el texto del boton si el raton pasa por encima
  if (texto){
    if (msg)  texto->colorTexto = colorRaton;
    else      texto->colorTexto = colorTexto;
  }

  // Se indica si ha sido pulsado
  return msg;
}

/**
 * Dibuja el boton en su posicion actual
 */
void Boton::dibujar()
{

  // Dibujar el widget
  Widget::dibujar();

  // Dibujar el texto
  if (texto) texto->dibujar();
}
