/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Mapa.h"
#include <stdio.h>
#include "Sistema.h"

#define FICHERO_NIVELES "niv/niveles.bin"

// Escenario estatico
ESCENA Mapa::escena[MAX_ALTO][MAX_LARGO][MAX_ANCHO];

/**
 * Constructor de la clase Mapa
 */
Mapa::Mapa(){
#ifdef DEBUG
  puts("Inicializando Mapa");
#endif

  // Cargar las imagenes que se usaran en el mapa.
  casilla[IMG_VACIO]     = new Casilla("img/bloque00.bmp");
  casilla[IMG_MARCAR]    = new Casilla("img/bloque01.bmp");
  casilla[IMG_SUELO]     = new Casilla("img/bloque02.bmp");
  casilla[IMG_PARED]     = new Casilla("img/bloque03.bmp");
  casilla[IMG_OBJETIVO]  = new Casilla("img/bloque04.bmp");
  casilla[IMG_CAJA]      = new Casilla("img/bloque05.bmp");
  casilla[IMG_CAJA2]     = new Casilla("img/bloque06.bmp");
  casilla[IMG_PERSONAJE] = new Casilla("img/personaje.bmp");

  // Preparar el mapa
  nNiveles=0;
  crearMapaVacio();

  // Incicializa valores por defecto a los atributos
  rX=-1;
  rY=-1;
  rZ=-1;

  jugando=1;
}

/**
 * Destructor de la clase mapa
 */
Mapa::~Mapa()
{

  if (casilla[IMG_VACIO])    delete casilla[IMG_VACIO];
  if (casilla[IMG_MARCAR])   delete casilla[IMG_MARCAR];
  if (casilla[IMG_SUELO])    delete casilla[IMG_SUELO];
  if (casilla[IMG_PARED])    delete casilla[IMG_PARED];
  if (casilla[IMG_OBJETIVO]) delete casilla[IMG_OBJETIVO];
  if (casilla[IMG_CAJA])     delete casilla[IMG_CAJA];

}

/**
 * Detecta sobre que casilla del mapa esta el raton
 */
void Mapa::detectarRaton(){
  int x, y, z;
  int pantX, pantY;
  // Por defecto, el raton empieza fuera del escenario
  rX=-1;
  rY=-1;
  rZ=-1;

  // Detectar donde esta el raton
  for (z=0;z<MAX_ALTO;z++){
    for (y=0;y<MAX_LARGO;y++){
      // Actualizar las posiciones isometricas
      pantX=500+(30*y);
      pantY=150+(15*y)-(12*z);
      for (x=0;x<MAX_ANCHO;x++){

        // Si la casilla tiene algo que dibujar,
        // o si es una casilla de base y no estamos jugando
        if ((escena[z][y][x].tipo) || ((!jugando) && (z==0)))
          // Si es el personaje, no marcar
          if (escena[z][y][x].tipo!=IMG_PERSONAJE)
            // Si es una pared y se esta jugando, no marcar
            if (!((escena[z][y][x].tipo==IMG_PARED) && (jugando)))
              // Detectar el raton
              if (casilla[escena[z][y][x].tipo]->detectarRaton(pantX,pantY))
              {
                rX=x; rY=y; rZ=z;
              }

        // Actualizar las posiciones isometricas
        pantX-=30;
        pantY+=15;
      }
    }
  }

}

/**
 * Ordena y prepara el mapa para ser dibujado
 */
void Mapa::ordenarProfundidad()
{
  int x, y, z;
  int pantX, pantY;
  int cont=0;
  int contAnt=0;
  int i, j, posmin, min, aux;

  // Por cada plano xy
  for (z=0;z<MAX_ALTO;z++){
    // Copiar todas las casillas con sus posiciones actuales a un array lineal
    for (y=0;y<MAX_LARGO;y++){
      // Actualizar las posiciones isometricas
      pantX=500+(30*y);
      pantY=150+(15*y)-(12*z);

      for (x=0;x<MAX_ANCHO;x++){
        // Copiar la informacion
        escOrdenada[cont].tipo = escena[z][y][x].tipo;
        escOrdenada[cont].desX = escena[z][y][x].desX + pantX;
        escOrdenada[cont].desY = escena[z][y][x].desY + pantY;

        // Cambiar de color la caja cuando esta sobre el objetivo
        if (z==1){
          if ((escena[1][y][x].tipo==IMG_CAJA) &&
              (escena[0][y][x].tipo==IMG_OBJETIVO))
              escOrdenada[cont].tipo = IMG_CAJA2;
        }

        if ((escOrdenada[cont].tipo) || ((!jugando) && (z==0)))
          escOrdenada[cont].ver=1;
        else
          escOrdenada[cont].ver=0;

        // Marcar la casilla si el raton esta aqui
        if ((rX==x) && (rY==y) && (rZ==z)){
          escOrdenada[cont].raton = RATON_DETECTADO;
        }
        else escOrdenada[cont].raton = 0;

        cont++;

        // Actualizar las posiciones isometricas
        pantX-=30;
        pantY+=15;
      }
    }

    // Ordenar el plano xy actual por el metodo de insercion
    for (i=contAnt; i<cont;i++){

      min=escOrdenada[i].desY;
      posmin=i;
      for (j=(i+1);j<cont;j++)
        if (escOrdenada[j].desY<min){
          min=escOrdenada[j].desY;
          posmin=j;
        }

      aux                       = escOrdenada[i].desY;
      escOrdenada[i].desY       = escOrdenada[posmin].desY;
      escOrdenada[posmin].desY  = aux;

      aux                       = escOrdenada[i].desX;
      escOrdenada[i].desX       = escOrdenada[posmin].desX;
      escOrdenada[posmin].desX  = aux;

      aux                       = escOrdenada[i].tipo;
      escOrdenada[i].tipo       = escOrdenada[posmin].tipo;
      escOrdenada[posmin].tipo  = aux;

      aux                       = escOrdenada[i].raton;
      escOrdenada[i].raton      = escOrdenada[posmin].raton;
      escOrdenada[posmin].raton = aux;

      aux                       = escOrdenada[i].ver;
      escOrdenada[i].ver        = escOrdenada[posmin].ver;
      escOrdenada[posmin].ver   = aux;
    }
    contAnt=cont;
  }
}

/**
 * Dibuja el mapa en pantalla
 */
void Mapa::dibujar()
{
  int TOTAL = MAX_ALTO * MAX_ANCHO * MAX_LARGO;
  //int x, y, z;
  //int pantX, pantY;
  int tipo, desX, desY;

  // Comprobar cual es la casilla sobre la que esta el raton
  detectarRaton();
  ordenarProfundidad();

  for (int i=0;i<TOTAL;i++){
    if ( (escOrdenada[i].ver) ){
      tipo = escOrdenada[i].tipo;
      desX = escOrdenada[i].desX;
      desY = escOrdenada[i].desY;

      // Dibujar los elementos del mapa
      casilla[tipo]->dibujar(desX, desY);

      // Si el raton esta sobre la casilla
      if (escOrdenada[i].raton){
        // Dibujar la marca sobre la casilla
        casilla[IMG_MARCAR]->dibujar(desX,desY
              - (casilla[tipo]->cuadro->al - 30) );
      }
    }
  }
}

/**
 * Cuenta las cajas que hay en el mapa
 * @return Devuelve el numero de cajas
 */
int Mapa::contarCajas()
{
  int x,y,nCajas=0;
 // Contar los objetos
  for (y=0;y<MAX_LARGO;y++)
    for (x=0;x<MAX_ANCHO;x++){
      if (escena[1][y][x].tipo == IMG_CAJA)       nCajas++;
    }
  return nCajas;
}

/**
 * Comprueba los objetivos del mapa
 * @return Devuelve el numero de cajas que hay sobre los objetivos
 */
int Mapa::comprobarObjetivos()
{
  int nCompletados=0;
  int x,y;
  for (y=0;y<MAX_LARGO;y++)
    for (x=0;x<MAX_ANCHO;x++)
      if ((escena[0][y][x].tipo == IMG_OBJETIVO) &&
          (escena[1][y][x].tipo == IMG_CAJA)) nCompletados++;

  return nCompletados;

}


/**
 * Comprueba que el mapa no tenga errores.
 * @return 1 = No hay un personaje
 *         2 = No hay cajas.
 *         3 = No hay tantas cajas como objetivos.
 *         4 = Todas las cajas estan sobre un objetivo
 *         5 = No se ha introducido un password
 */
int Mapa::validarMapa()
{
  int nObjetivos=0;
  int nCajas=0;
  int nPersonajes=0;
  int x,y;

  // Contar los objetos
  for (y=0;y<MAX_LARGO;y++)
    for (x=0;x<MAX_ANCHO;x++){
      if (escena[0][y][x].tipo == IMG_OBJETIVO)   nObjetivos++;
      if (escena[1][y][x].tipo == IMG_CAJA)       nCajas++;
      if (escena[1][y][x].tipo == IMG_PERSONAJE)  nPersonajes++;
    }

  // Comprobar errores
  //
  // Si no hay un personaje
  if (nPersonajes!=1)               return 1;

  // Si no hay cajas
  if (nCajas<=0)                    return 2;

  // Si no hay tantas cajas como objetivos
  if (nCajas!=nObjetivos)           return 3;

  // Si todas las cajas estan en su objetivo
  if (comprobarObjetivos()==nCajas) return 4;

  // Si no hay password
  if (strlen(password)==0)          return 5;

  // Mapa validado correctamente
  return 0;
}

/**
 * Prepara un mapa vacio
 */
void Mapa::crearMapaVacio()
{
  int x,y,z,cont=0;

  for (z=0;z<MAX_ALTO;z++)
    for (y=0;y<MAX_LARGO;y++)
      for (x=0;x<MAX_ANCHO;x++){

        escena[z][y][x].tipo    = 0;
        escena[z][y][x].desX    = 0;
        escena[z][y][x].desY    = 0;

        escOrdenada[cont].tipo  = 0;
        escOrdenada[cont].desX  = 0;
        escOrdenada[cont].desY  = 0;
        escOrdenada[cont].raton = 0;
        escOrdenada[cont].ver   = 0;

        cont++;
      }

  puntos  [0]                   = 0;
  pasos   [0]                   = 0;
  password[0]                   = 0;

  nNiveles++;
}

/**
 * Carga un mapa desde disco por numero de nivel
 * @param nivel Nivel que se va a cargar
 * @return Devuelve el numero de nivel cargado
 *         o menor que 0 si hubo un error
 */
int Mapa::cargar(int nivel)
{
  int resultado=0;
  FILE* fichero;

  int tamEscena = sizeof(ESCENA) * (MAX_ALTO * MAX_LARGO * MAX_ANCHO);
  int tamNivel  = tamEscena + sizeof(puntos) + sizeof(pasos) + sizeof(password);

  // Hacer que no se pueda poner un nivel que no existe
  if (nivel<0) return -3;

  // Cargar el fichero de niveles
  fichero=fopen(FICHERO_NIVELES,"rb");
  if (!fichero) return -1;

  // Calcular el numero de niveles que hay en el fichero
  fseek(fichero,0,SEEK_END);
  nNiveles=ftell(fichero)/tamNivel;

  // Si se solicita un nivel superior al numero de niveles, volver
  if (nivel>=nNiveles) return -2;
  if (nivel<0) return -3;
  resultado=nivel;

  // Ir a la posicin correspondiente al nivel
  fseek(fichero, tamNivel * nivel, SEEK_SET);

  // Copiar los datos del nivel al fichero
  fread ( &puntos,   sizeof(puntos),   1, fichero);
  fread ( &pasos,    sizeof(password), 1, fichero);
  fread ( &password, sizeof(password), 1, fichero);
  fread ( &escena,   tamEscena,        1, fichero);

  fclose(fichero);

  return resultado;
}

/**
 * Carga un mapa desde disco por su password
 * @param pass Password del nivel que se va a cargar
 * @return Devuelve el numero de nivel cargado
 *         o menor que 0 si hubo un error
 */
int Mapa::cargar(char* pass)
{
  int i;
  FILE* fichero;

  int tamEscena = sizeof(ESCENA) * (MAX_ALTO * MAX_LARGO * MAX_ANCHO);
  int tamNivel  = tamEscena + sizeof(puntos) + sizeof(pasos) + sizeof(password);

  // Cargar el fichero de niveles
  fichero=fopen(FICHERO_NIVELES,"rb");
  if (!fichero) {
    puts("Error al cargar el fichero de niveles");
    return -1;
  }

  // Calcular el numero de niveles que hay en el fichero
  fseek(fichero,0,SEEK_END);
  nNiveles=ftell(fichero)/tamNivel;
  fclose(fichero);

  for (i=0;i<nNiveles;i++){
    // Cargar el fichero de niveles
    fichero=fopen(FICHERO_NIVELES,"rb");
    if (!fichero) return -1;
    // Ir a la posicin correspondiente al nivel
    fseek(fichero, tamNivel * i, SEEK_SET);
    // Copiar los datos del nivel al fichero
    fread ( &puntos,   sizeof(puntos),   1, fichero);
    fread ( &pasos,    sizeof(password), 1, fichero);
    fread ( &password, sizeof(password), 1, fichero);
    fclose(fichero);
    if (strcmp(pass,password)==0) return cargar(i);
  }
  pasos[0]=0;
  puntos[0]=0;
  password[0]=0;
  return -4;
}

/**
 * Guarda el nivel en disco en la posicion de nivel indicada
 * @param nivel Posicion (nivel) que se quiere sobreescribir
 * @return 0 si se guardo correctamente
 *         distinto de cero si hubo un error de escritura
 */
int Mapa::guardar(int nivel)
{
  FILE* fichero;

  int tamEscena = sizeof(ESCENA) * (MAX_ALTO * MAX_LARGO * MAX_ANCHO);
  int tamNivel  = tamEscena + sizeof(puntos) + sizeof(pasos) + sizeof(password);

  // Tratar de abrir el fichero para aadir
  fichero=fopen(FICHERO_NIVELES,"r+b");

  // Si no se pudo abrir, tratar de crearlo
  if (!fichero) {
    puts("Error: No se pudo abrir el fichero de mapas para escritura");
    fichero=fopen(FICHERO_NIVELES,"wb");
    // Si no se pudo crear, volver
    if (!fichero){
      puts("Error: no se pudo crear el fichero de mapas");
      return 1;
    }
  }
  // Si se pudo abrir, ir a la posicin correspondiente al nivel
  else fseek(fichero, tamNivel * nivel, SEEK_SET);

  // Copiar los datos del nivel al fichero
  fwrite ( &puntos,   sizeof(puntos),   1, fichero);
  fwrite ( &pasos,    sizeof(password), 1, fichero);
  fwrite ( &password, sizeof(password), 1, fichero);
  fwrite ( &escena,   tamEscena,        1, fichero);

  fclose(fichero);
  return 0;
}

