/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Principal.h"
#include <stdio.h>

/**
 * Instancia las clases de los subprogramas.
 */
Principal::Principal()
{
#ifdef DEBUG
  puts("Inicializando Principal");
#endif

  // Inicializar el sistema
  sistema = new Sistema();

  // Cargar y prerarar los textos
  Programa::prepararTexto();

  // Crear los objetos de los subprogramas
  menu    = new Menu();
  juego   = new Juego();
  editor  = new Editor();
  records = new Records();
}

/**
 * Destruye todos los objetos instanciados en el constructor
 */
Principal::~Principal()
{
  // Destruir los objetos instanciados por "principal"
  if (records)  delete records;
  if (editor)   delete editor;
  if (juego)    delete juego;
  if (menu)     delete menu;
  if (sistema)  delete sistema;
}

/**
 * Bucle principal en el que se gestiona todo el programa.
 */
void Principal::bucle()
{
#ifdef DEBUG
  puts("Entra en bucle principal");
#endif


  bool salir=0;

  // Repetir ...
  do{

    // El menu solicita una opcion al usuario
    switch (menu->bucle()){
      // Opcion 1: Jugar
      case OPC_JUGAR:     juego->bucle(JUEGO_NUEVO);      break;

      // Opcion 2: Continuar con password
      case OPC_CONTINUAR: juego->bucle(JUEGO_CONTINUAR);  break;

      // Opcion 3: Editar niveles
      case OPC_EDITOR:    puts("Editor"); editor->bucle();                break;

      // Opcion 4: Tabla de records
      case OPC_RECORDS:   records->bucle();               break;

      // Opcion 5: Salir
      case OPC_SALIR:     salir=1;                        break;
    }

  // ... hasta que se solicite salir
  }while(!salir);
}
