/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Programa.h"
#include "Sistema.h"
#include "Boton.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

Texto**      Programa::texto= 0;
INFOTEXTO*  Programa::infoTexto = 0;
int         Programa::nTextos   = 0;

/**
 * Constructor de la clase programa
 * Esta clase no hace nada por si sola. Esta pensada para ser sobreescrita
 * por las clases Menu, Juego, Editor y Records.
 */
Programa::Programa(){
  // Inicializa los atributos por defecto
  m_salir=0;
  m_pausa=200;
  nTextos=0;
}

/**
 * Destructor de la clase programa
 */
Programa::~Programa(){

}
/**
 * Comprobaciones de entrada generales para todos los subprogramas
 */
void Programa::comprobarEntrada()
{
  if (Sistema::comprobarTeclado(TECLA_ESC)) m_salir=-1;
}

/**
 * Comprueba el estado del programa
 */
void Programa::comprobarEstado()
{
  if (m_salir==-1) terminar();

}

/**
 * Actualizacion de la pantalla
 * Este metodo debe ser sobreescrito
 */
void Programa::actualizarPantalla()
{

}

/**
 * Termina el programa (tras confirmacion del usuario)
 * @return Devuelve la respuesta del usuario.
 */
int Programa::terminar()
{
  if (mostrarMensaje(TXT_SOKONUX,TXT_SALIR,0)==MSG_ACEPTAR)
    m_salir=-1;
  else
    m_salir=0;

  return m_salir;
}

/**
 * Bucle principal de los subprogramas
 */
int Programa::bucle()
{
  m_salir=0;
  Sistema::borrarPantallaOculta(0);

  do{
    comprobarEntrada();
    comprobarEstado();
    actualizarPantalla();
    Sistema::volcarPantalla();
  }while(!m_salir);

  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();
  return m_salir;
}

/**
 * Crea una ventana sencilla con barra de titulo y un mensaje centrado
 * Este metodo es usado por los metodos pedirInformacion y mostrarMensaje
 * para crear sus ventanas.
 * @param titulo  Texto de la barra de titulo de la ventana
 * @param mensaje Texto que se mostrara dentro de la ventana
 * @param ancho Anchura de la ventana
 * @param alto Altura de la ventana
 */
Ventana* Programa::crearVentana(Texto* titulo, Texto* mensaje,
                                int ancho, int alto)
{
  // Crear un cuadro de texto
  Ventana *ventana=new Ventana(0,titulo,0,0,ancho,alto,DF_FLAGSVENTANA);
  ventana->centrar(WID_CENTRARX|WID_CENTRARY);

  if (mensaje){
    // Agregar el texto
    Texto* t = ventana->agregarTexto(mensaje, 10, 15, 0xFFFFFFFF, ARIAL12 );

    t->centrar(WID_CENTRARX);
  }

  return ventana;
}

/**
 * Muestra una ventana con un cuadro de texto para pedir informacion al usuario
 * @param titulo  Texto de la barra de titulo de la ventana
 * @param mensaje Texto que se mostrara para pedir la informacion
 * @param respuesta Puntero a la cadena donde se almacenara la respuesta
 * @return Devuelve el boton que ha pulsado el usuario
 *          MSG_ACEPTAR o MSG_CANCELAR
 */
int Programa::pedirInformacion(Texto* titulo, Texto* mensaje,
                               char **respuesta, int flags)
{
  int respBoton=0;
  bool confirmado=0;
  Boton *aceptar, *cancelar;
  CuadroTexto *info;

  Ventana *cuadroTexto = crearVentana(titulo,mensaje,250,140);

  // Crear los botones
  info     = cuadroTexto->agregarCuadroTexto ( 20, 40, 100, 22, 10 );
  aceptar  = cuadroTexto->agregarBoton       ( TXT_ACEPTAR, 10, 80, 60, 30 );
  cancelar  = cuadroTexto->agregarBoton      ( TXT_CANCELAR, 180, 80, 60, 30 );

  // Vaciar el buffer del teclado
  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();

  // Bucle de la ventana
  do{
    actualizarPantalla();
    cuadroTexto->dibujar();
    Sistema::volcarPantalla();

    // Si se pulsa aceptar
    if ( (aceptar->detectarRaton()==MSG_RATON_CLICK) ||
         (Sistema::comprobarTeclado(TECLA_RET)) )
    {
      respBoton=MSG_ACEPTAR;
      confirmado=1;
    }

    // Si se pulsa cancelar
    if ( (cancelar->detectarRaton()==MSG_RATON_CLICK) ||
         (Sistema::comprobarTeclado(TECLA_ESC)) )
    {
      respBoton=MSG_CANCELAR;
      confirmado=1;
    }
  // Salir cuando se haya confirmado
  }while(!confirmado);

  // Vaciar buffer del teclado
  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();

  // Copiar la respuesta a la cadena indicada
  *respuesta = new char[strlen(info->textoEscrito)+1];
  strcpy(*respuesta,info->textoEscrito);

  delete cuadroTexto;

  return respBoton;
}

/**
 * Muestra una ventana de informacion al usuario
 * @param titulo  Texto de la barra de titulo de la ventana
 * @param mensaje Texto que se mostrara al usuario
 * @param flags Botones que se mostraran (0 = aceptar y cancelar:
                MSG_ACEPTAR, MSG_SINO = si/no, MSG_SINOCAN = Si, no y cancelar)
 * @return Devuelve el boton que ha pulsado el usuario
 *          MSG_ACEPTAR, MSG_CANCELAR o MSG_NO
 */
int Programa::mostrarMensaje(Texto* titulo, Texto* mensaje, int flags)
{
  int respuesta=0;
  bool confirmado=0;
  Boton *aceptar=0, *cancelar=0, *no=0;

  Ventana *cuadroTexto = crearVentana(titulo,mensaje,250,110);

  // Crear los botones segun los flags
  switch (flags){
    case MSG_ACEPTAR:
      aceptar  = cuadroTexto->agregarBoton  (TXT_ACEPTAR,95,50,60,30);
      cancelar = 0;
      break;
    case MSG_SINO:
      aceptar  = cuadroTexto->agregarBoton  (TXT_SI,10,50,60,30);
      cancelar = cuadroTexto->agregarBoton  (TXT_NO,180,50,60,30);
      break;

    case MSG_SINOCAN:
      aceptar  = cuadroTexto->agregarBoton  (TXT_SI,10,50,60,30);
      no       = cuadroTexto->agregarBoton  (TXT_NO,95,50,60,30);
      cancelar = cuadroTexto->agregarBoton  (TXT_CANCELAR,180,50,60,30);
      break;

    default:
      aceptar  = cuadroTexto->agregarBoton  (TXT_ACEPTAR,10,50,60,30);
      cancelar = cuadroTexto->agregarBoton  (TXT_CANCELAR,180,50,60,30);
  }

  // Vaciar el buffer del teclado
  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();

  // Bucle de la ventana
  do{
    actualizarPantalla();
    cuadroTexto->dibujar();
    Sistema::volcarPantalla();

    // Si se pulsa aceptar
    if ( (aceptar->detectarRaton()==MSG_RATON_CLICK) ||
         (Sistema::comprobarTeclado(TECLA_RET)) )
    {
      respuesta=MSG_ACEPTAR;
      confirmado=1;
    }

    // Si se pulsa cancelar
    if (cancelar)
      if ( (cancelar->detectarRaton()==MSG_RATON_CLICK) ||
         (Sistema::comprobarTeclado(TECLA_ESC)) )
      {
        respuesta=MSG_CANCELAR;
        confirmado=1;
      }

    // Si se pulsa "no"
    if (no)
      if ( (no->detectarRaton()==MSG_RATON_CLICK) ||
         (Sistema::comprobarTeclado(TECLA_ESC)) )
      {
        respuesta=MSG_NO;
        confirmado=1;
      }
  // Repetir hasta que se pulse un boton
  }while(!confirmado);

  // Vaciar buffer del teclado
  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();

  delete cuadroTexto;

  return respuesta;
}


/**
 * Prepara los textos del programa a partir del fichero de texto
 * Definido en la constante FICHERO_TEXTOS en archivo "Texto.h"
 */
void Programa::prepararTexto(){

  nTextos = Sistema::cargarTexto(FICHERO_TEXTOS,"[PROGRAMA]",&infoTexto);
  texto = new Texto*[nTextos];

  for (int i=0;i<nTextos;i++)
    texto[infoTexto[i].id] = new Texto(0,infoTexto[i]);

}
