/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Sistema.h"
#include "Texto.h"

/**
 * Constructor de la clase texto a partir de una estructura infoTexto
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param infoTexto   Estructura con la informacin del texto que se va a crear
 */
Texto::Texto(CUADRO** cuadroPadre, INFOTEXTO infoTexto)
     : Widget(cuadroPadre, infoTexto.x, infoTexto.y, 0, 0)
{
  // Copiar la informacion del texto
  cadena      = infoTexto.cadena;
  colorTexto  = infoTexto.colorTexto;
  colorSombra = infoTexto.colorSombra;
  colorRaton  = infoTexto.colorRaton;
  despSombra  = infoTexto.despSombra;
  flags       = infoTexto.flags;

  if (!infoTexto.fuente)  fuente = DF_FUENTE;
  else                    fuente = infoTexto.fuente;

  // Obtener las dimensiones del cuadro para el texto
  Sistema::obtenerTamTexto(cadena,fuente, &this->cuadro->an, &this->cuadro->al);
}

/**
 * Destructor de la clase texto
 */
Texto::~Texto()
{

}

/**
 * Dibuja el texto en su posicion acutal, con los parametros actuales
 */
void Texto::dibujar()
{
  // Obtener las dimensiones del cuadro para el texto
  Sistema::obtenerTamTexto(cadena,fuente, &this->cuadro->an, &this->cuadro->al);

  centrar(flags);

  // Dibujar la sombra
  if (flags&WID_SOMBRA){
    Sistema::escribirTexto (cadena, xAbs(cuadro)+despSombra,
                            yAbs(cuadro)+despSombra, colorSombra, fuente);
  }

  // Si se detecta el raton
  if ((flags&WID_DETECTARRATON) && (detectarRaton()))
    // Dibujar la imagen para cuando pasa el raton
    Sistema::escribirTexto (cadena, xAbs(cuadro), yAbs(cuadro),
                            colorRaton, fuente);
    else
    // Dibujar la imagen normal
    Sistema::escribirTexto (cadena, xAbs(cuadro), yAbs(cuadro),
                            colorTexto, fuente);
}

/**
 * Escribe el texto con formato. Similar a la orden printf, pero usando
 * Este texto como formato e indicando los valores para el texto por parametro
 * @param param Reservado. Siempre debe ser 0.
 * @param ... lista de variables para el texto
 */
void Texto::dibujarFormateado(int param, ...)
{
  INFOTEXTO tmpInfoTexto={0,
                          new char[1000],
                          cuadro->x, cuadro->y,
                          colorTexto,
                          colorSombra,
                          colorRaton,
                          fuente,
                          despSombra,
                          flags};
  va_list varList;
  va_start(varList, param);
  vsprintf(tmpInfoTexto.cadena, cadena, varList);

  Texto* resultado=new Texto(0,tmpInfoTexto);
  if (resultado) {
    resultado->dibujar();
    delete resultado;
  }
}
